==> peekabot-0.6.0.tar.bz2: peekabot-0.6.0/src/atl/UnauthenticatedState.cc <==
/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <boost/bind.hpp>

#include "ConnectionStates.hh"
#include "../Bytesex.hh"
#include "../Protocol.hh"


using namespace peekabot;
using namespace peekabot::atl;


UnauthenticatedState::UnauthenticatedState(ClientConnection *connection) throw()
    : ConnectionState(connection), 
      m_auth_res_local((uint8_t)protocol::AUTH_SUCCEEDED),
      m_auth_res_remote((uint8_t)protocol::AUTH_SUCCEEDED),
      m_close_if_outbound_empty(false)
{
}


UnauthenticatedState::~UnauthenticatedState() throw()
{
}


void UnauthenticatedState::on_data_available() throw()
{
    // TODO: to make this more efficient, we could
    // utilize ClientConnection::peek to process
    // more than one read record at once.

    if( !m_read_queue.empty() )
    {
        ReadRecord &rr = m_read_queue.front();

        rr.m_n_read += get_connection()->recv(
            rr.m_dest, rr.m_size - rr.m_n_read);

        if( rr.m_n_read >= rr.m_size )
        {
            // Data in read record read in its entirety,
            // run the callback and continue with the next read record

            // Pop the read record
            ReadRecord tmp = m_read_queue.front();
            m_read_queue.pop();

            // ...and THEN call the callback
            tmp.m_callback();
        }
    }
}



void UnauthenticatedState::on_tick(boost::posix_time::time_duration &dt) throw()
{
    if( get_connection()->get_uptime().total_milliseconds() >=
        protocol::AUTHENTICATION_TIMEOUT )
    {
        get_connection()->close(ClientConnection::AUTHENTICATION_TIMED_OUT);
    }
    else if( m_close_if_outbound_empty && 
             get_connection()->get_outbound_count() == 0 )
    {
        get_connection()->close(ClientConnection::AUTHENTICATION_FAILED);
    }
}


void UnauthenticatedState::on_state_active() throw()
{
    // Send authentication data
    uint8_t is_big_endian = (PEEKABOT_BYTE_ORDER == PEEKABOT_BIG_ENDIAN);

    get_connection()->send(&is_big_endian, 
                           sizeof(uint8_t));
    get_connection()->send(&protocol::UNIQUE_ID, 
                           sizeof(protocol::UNIQUE_ID));
    get_connection()->send(&protocol::PROTOCOL_VERSION, 
                           sizeof(protocol::PROTOCOL_VERSION));
    get_connection()->send(&version::PEEKABOT_VERSION, 
                           sizeof(version::PEEKABOT_VERSION));

    // "Read" incoming handshaking data
    read(&m_is_big_endian, sizeof(uint8_t));
    read(&m_unique_id, sizeof(m_unique_id));
    read(&m_protocol_version, sizeof(m_protocol_version));
    read(&m_peekabot_version, sizeof(m_peekabot_version));
    read(&m_enable_low_latency_mode, sizeof(m_enable_low_latency_mode),
         boost::bind(&UnauthenticatedState::part_one_read, this));
}



void UnauthenticatedState::read(void *dest, size_t n) throw()
{
    read(dest, n, UnauthenticatedState::null_callback);
}

void UnauthenticatedState::read(
    void *dest, size_t n, ReadCallback callback) throw()
{
    ReadRecord rr;

    rr.m_dest = dest;
    rr.m_size = n;
    rr.m_n_read = 0;
    rr.m_callback = callback;

    m_read_queue.push(rr);
}






void UnauthenticatedState::part_one_read() throw()
{
    // Swap byte order of read data if needed:
    if( m_is_big_endian != (PEEKABOT_BYTE_ORDER == PEEKABOT_BIG_ENDIAN) )
    {
        switch_byte_order(&m_unique_id, 1);
        switch_byte_order(&m_protocol_version, 1);
        switch_byte_order(&m_peekabot_version, 1);
    }

    get_connection()->toggle_low_latency_mode(m_enable_low_latency_mode != 0);

    // Verify part one data:
    if( m_unique_id != protocol::UNIQUE_ID )
        // Invalid unique ID received
        m_auth_res_local = protocol::AUTH_SERVER_RECEIVED_UNEXPECTED_DATA;
    else if( m_protocol_version != protocol::PROTOCOL_VERSION )
        // Incompatible protocol versions
        m_auth_res_local = protocol::AUTH_INCOMPAT_PROTOCOL_VERSION;
    else if( m_peekabot_version < version::PEEKABOT_COMPATIBLE_VERSION )
        // Incompatible peekabot versions
        m_auth_res_local = protocol::AUTH_INCOMPAT_PEEKABOT_VERSION;

    // Send auth response to client
    get_connection()->send(&m_auth_res_local, sizeof(uint8_t));

    // Read auth response from client
    read(&m_auth_res_remote, sizeof(uint8_t),
         boost::bind(&UnauthenticatedState::part_two_read, this));
}


void UnauthenticatedState::part_two_read() throw()
{
    // Switch to Authenticated if local and remote 
    // authentication results indicate success
    if( m_auth_res_remote != protocol::AUTH_SUCCEEDED ||
        m_auth_res_local != protocol::AUTH_SUCCEEDED )
    {
        // If authentication failed, signal the state to shutdown the 
        // connection once the outbound buffer is empty
        m_close_if_outbound_empty = true;
    }
    else
    {
        set_proxy_version((m_peekabot_version & 0x00FF0000) >> 16,
                          (m_peekabot_version & 0x0000FF00) >> 8,
                          (m_peekabot_version & 0x000000FF) >> 0);
        set_protocol_version(m_protocol_version);
        
        
        // Change connection state to Authenticated
        get_connection()->set_state(
            new AuthenticatedState(
                get_connection(), m_is_big_endian));
    }
}

==> peekabot-0.7.2.tar.bz2: peekabot-0.7.2/src/atl/UnauthenticatedState.cc <==
/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <boost/bind.hpp>

#include "ConnectionStates.hh"
#include "../Bytesex.hh"
#include "../Protocol.hh"
#include "../Version.hh"


using namespace peekabot;
using namespace peekabot::atl;


UnauthenticatedState::UnauthenticatedState(ClientConnection *connection) throw()
    : ConnectionState(connection), 
      m_auth_res_local((uint8_t)protocol::AUTH_SUCCEEDED),
      m_auth_res_remote((uint8_t)protocol::AUTH_SUCCEEDED),
      m_close_if_outbound_empty(false)
{
}


UnauthenticatedState::~UnauthenticatedState() throw()
{
}


void UnauthenticatedState::on_data_available() throw()
{
    // TODO: to make this more efficient, we could
    // utilize ClientConnection::peek to process
    // more than one read record at once.

    if( !m_read_queue.empty() )
    {
        ReadRecord &rr = m_read_queue.front();

        rr.m_n_read += get_connection()->recv(
            rr.m_dest, rr.m_size - rr.m_n_read);

        if( rr.m_n_read >= rr.m_size )
        {
            // Data in read record read in its entirety,
            // run the callback and continue with the next read record

            // Pop the read record
            ReadRecord tmp = m_read_queue.front();
            m_read_queue.pop();

            // ...and THEN call the callback
            tmp.m_callback();
        }
    }
}



void UnauthenticatedState::on_tick(boost::posix_time::time_duration &dt) throw()
{
    if( get_connection()->get_uptime().total_milliseconds() >=
        protocol::AUTHENTICATION_TIMEOUT )
    {
        get_connection()->close(ClientConnection::AUTHENTICATION_TIMED_OUT);
    }
    else if( m_close_if_outbound_empty && 
             get_connection()->get_outbound_count() == 0 )
    {
        get_connection()->close(ClientConnection::AUTHENTICATION_FAILED);
    }
}


void UnauthenticatedState::on_state_active() throw()
{
    // Send authentication data
    uint8_t is_big_endian = (PEEKABOT_BYTE_ORDER == PEEKABOT_BIG_ENDIAN);

    get_connection()->send(&is_big_endian, 
                           sizeof(uint8_t));
    get_connection()->send(&protocol::UNIQUE_ID, 
                           sizeof(protocol::UNIQUE_ID));
    get_connection()->send(&protocol::PROTOCOL_VERSION, 
                           sizeof(protocol::PROTOCOL_VERSION));
    uint32_t ver = PEEKABOT_VERSION | (PEEKABOT_RELEASE_STATUS << 24);
    get_connection()->send(&ver, sizeof(uint32_t));

    // "Read" incoming handshaking data
    read(&m_is_big_endian, sizeof(m_is_big_endian));
    read(&m_unique_id, sizeof(m_unique_id));
    read(&m_protocol_version, sizeof(m_protocol_version));
    read(&m_peekabot_version, sizeof(m_peekabot_version));
    read(&m_enable_low_latency_mode, sizeof(m_enable_low_latency_mode),
         boost::bind(&UnauthenticatedState::part_one_read, this));
}



void UnauthenticatedState::read(void *dest, size_t n) throw()
{
    read(dest, n, UnauthenticatedState::null_callback);
}

void UnauthenticatedState::read(
    void *dest, size_t n, ReadCallback callback) throw()
{
    ReadRecord rr;

    rr.m_dest = dest;
    rr.m_size = n;
    rr.m_n_read = 0;
    rr.m_callback = callback;

    m_read_queue.push(rr);
}






void UnauthenticatedState::part_one_read() throw()
{
    // Swap byte order of read data if needed:
    if( m_is_big_endian != (PEEKABOT_BYTE_ORDER == PEEKABOT_BIG_ENDIAN) )
    {
        switch_byte_order(&m_unique_id, 1);
        switch_byte_order(&m_protocol_version, 1);
        switch_byte_order(&m_peekabot_version, 1);
    }

    get_connection()->toggle_low_latency_mode(m_enable_low_latency_mode != 0);

    // Verify part one data:
    if( m_unique_id != protocol::UNIQUE_ID )
        // Invalid unique ID received
        m_auth_res_local = protocol::AUTH_SERVER_RECEIVED_UNEXPECTED_DATA;
    else if( m_protocol_version != protocol::PROTOCOL_VERSION )
        // Incompatible protocol versions
        m_auth_res_local = protocol::AUTH_INCOMPAT_PROTOCOL_VERSION;
    else if( (m_peekabot_version & 0x00FFFFFF) < PEEKABOT_COMPATIBLE_VERSION )
        // Incompatible peekabot versions
        m_auth_res_local = protocol::AUTH_INCOMPAT_PEEKABOT_VERSION;

    // Send auth response to client
    get_connection()->send(&m_auth_res_local, sizeof(uint8_t));

    // Read auth response from client
    read(&m_auth_res_remote, sizeof(uint8_t),
         boost::bind(&UnauthenticatedState::part_two_read, this));
}


void UnauthenticatedState::part_two_read() throw()
{
    // Switch to Authenticated if local and remote 
    // authentication results indicate success
    if( m_auth_res_remote != protocol::AUTH_SUCCEEDED ||
        m_auth_res_local != protocol::AUTH_SUCCEEDED )
    {
        // If authentication failed, signal the state to shutdown the 
        // connection once the outbound buffer is empty
        m_close_if_outbound_empty = true;
    }
    else
    {
        set_client_version((m_peekabot_version & 0x00FF0000) >> 16,
                           (m_peekabot_version & 0x0000FF00) >> 8,
                           (m_peekabot_version & 0x000000FF) >> 0,
                           (m_peekabot_version & 0xFF000000) >> 24);
        set_protocol_version(m_protocol_version);

        // Change connection state to Authenticated
        get_connection()->set_state(
            new AuthenticatedState(
                get_connection(), m_is_big_endian));
    }
}
